
# Changelog v0.0.4

## Highlights

### New Features & Enhancements

#### Refactor Source Code Structure
- **Merged** the ITS and Graph subpackages for improved modularity.
- **Separated** `its_decompose` and `get_rc` into a dedicated Python file: `its_decompose.py`.
- **Created** a new `Utils` subpackage for future utility functions and support tools.

#### Performance Improvements
- **Optimized** the `BatchCluster` process for faster clustering of reaction centers.

#### Bug Fixes
- **Fixed** issues with *Subgraph* searching: `networkx` now handles both *monomorphism* and *induced subgraph*, whereas `GML` was limited to *monomorphism* only.

#### New Features
- **Implemented** `CRN` (Chemical Reaction Networks) in the `Reactor` module.
- **Implemented** `PathFinder` to find pathways from a starting compound to the target compound in `CRN`.
- **Introduced** `RetroReactor` in the `Rule` module, enabling the use of A* algorithm to search from target to starting compound without requiring a full system like `CRN`.

## Known Limitations

### Rule Composition
- **Bug** in the valence constraint of rule composition.
- **Partial matching** in rule composition may result in invalid compounds.

### Retro
- **Directed CRN** can only find one solution, though it is 500 times faster than expanding CRN and using `PathFinder`.
- **Retro framework** using rule composition may result in invalid compounds.

### Reactor
- `ReactorEngine` currently only supports **balanced reactions**.
- **Performance issue**: `ReactorEngine` is slow in certain cases.
- **Memory leak** in `CoreEngine` when scaled up.




# Changelog v0.0.3

## Highlights

### New Features & Enhancements

#### Reduce Dependency and Update Version
- Removed dependency on `drfp` and `rxnmapper`.
- Updated versions of `numpy` and `torch` to 2.2.0.

#### Enhance `Reactor`
- `CoreEngine` and `ReactorEngine` have consistent function `_inference`.
- `CoreEngine` is refractored to handle reagents.

#### Add Visualization
- Added visualization for Double Pushout rules.

#### Graph Isomorphism and Subgraph Isomorphism
- Separated graph and rule morphism.
- Implemented subgraph isomorphism in `networkx` and `gml`.
- Added WL hash function to filter graphs before doing isomorphism check; does not work for subgraph isomorphism.

#### Chemical Conversion
- Added function to add implicit and explicit hydrogen.
- Enhanced conversion capabilities, now can handle both implicit and explicit hydrogen effectively.
- Added new functions for conversion between ITS and GML.

#### Atom-Atom Map
- Added `FixAAM` to handle AAM generated by MOD.
- `ReactorEngine` now works well to generate complete AAM from partial AAM, but only in good AAM cases.
- Fixed `Normalize` class issue of removing hydrogen without adding the implicit hydrogen to the graph. Now it is compatible with RDKit.

#### Rule
- Fixed issue of memory leakage when doing rule composition by adding argument `add=False` to `ruleGMLString`.

## Known Limitations

### BatchCluster
- The processing speed of `BatchCluster` is slower compared to `GraphCluster`. There may be an internal issue affecting performance.

### Reactor
- `ReactorEngine` currently only works with balanced reactions.
- `ReactorEngine` performs slowly in a few cases.
- `CoreEngine` exhibited memory leak when scaled up.

### Subgraph Isomorphism
- For `networkx` version, it performs very fast with filter, ~0.7s for searching 1 subgraph along with 33,000 graphs, however, the result is not correct ~330 subgraphs are incorrect.
- For `GML` version, it performs perfectly, but slow ~8s for searching 1 subgraph along with 33,000 graphs.
